## $Id: student.py 18036 2025-03-11 19:29:27Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Container for the various objects owned by students.
"""
import grok
from zope.interface import implementedBy
from waeup.kofa.utils.helpers import attrs_to_fields
from waeup.kofa.students.student import StudentFactory
from waeup.kofa.students.interfaces import IStudentNavigation
from waeup.kofa.utils.helpers import get_current_principal
from kofacustom.nigeria.students.student import NigeriaStudent
from kofacustom.iuokada.students.interfaces import (
    ICustomStudent, ICustomStudentPersonalEdit)


class CustomStudent(NigeriaStudent):
    """This is a student container for the various objects
    owned by students.
    """
    grok.implements(ICustomStudent, IStudentNavigation,
        ICustomStudentPersonalEdit)
    grok.provides(ICustomStudent)

    @property
    def transcript_enabled(self):
        user = get_current_principal()
        if user.id in ('admin', 'isouaba', 'niyi', 'delejason'):
            return True
        return False

    @property
    def is_jupeb(self):
        if self.faccode.startswith('JUPEB'):
            return True
        return False

    def minimumStudentPayments_not_used(self, minamount):
        """Need to access edit_personal. Only in iuokada package.
        """
        #if not self.is_fresh:
        #    return False
        if self.sponsor in ['bauchi', 'federalgov', 'kano']:
            return True
        total_amount = 0.0
        for ticket in self['payments'].values():
            if ticket.p_state == 'paid' \
                and ticket.p_session >= self.current_session:
                total_amount += ticket.amount_auth
        if total_amount >= minamount:
            return True
        return False

    def minimumStudentPayments(self):
        """Need to access edit_personal. Only in iuokada package.
        """
        if self.sponsor in ['bauchi', 'federalgov', 'kano']:
            return True
        if self.is_jupeb:
            return True
        try:
            certificate = self['studycourse'].certificate
        except (AttributeError, TypeError):
            return _('Study course data are incomplete.'), None
        total_amount = 0.0
        for ticket in self['payments'].values():
            if ticket.p_state == 'paid' \
                and ticket.p_session >= self.current_session \
                and ticket.p_category in ('schoolfee', 'schoolfee40',
                                          'secondinstal', 'clearance',):
                total_amount += ticket.amount_auth
        try:
            if self.is_fresh:
                minamount = 0.4 * getattr(certificate, 'school_fee_1', 0.0)
            else:
                minamount = 0.4 * getattr(certificate, 'school_fee_2', 0.0)
            if total_amount >= minamount:
                return True
        except:
            pass
        return False

    @property
    def personal_data_expired(self):
        # This reqirement is not compatible with minimumFreshStudentPayments.
        return False


# Set all attributes of Student required in IStudent as field
# properties. Doing this, we do not have to set initial attributes
# ourselves and as a bonus we get free validation when an attribute is
# set.
CustomStudent = attrs_to_fields(CustomStudent)

class CustomStudentFactory(StudentFactory):
    """A factory for students.
    """

    def __call__(self, *args, **kw):
        return CustomStudent()

    def getInterfaces(self):
        return implementedBy(CustomStudent)